/*
 * This is a part of the users_csv plugin for using with PluXml cms
 * author: Jean-Pierre Pourrez aka bazooka07
 * date: 2017-03-15
 *
 * */

(function(idTable, idSelection) {

	// add one row at the end of the table on each click

	usersTable = document.getElementById(idTable);

	if(usersTable != null) {
		var
			rowsCount = usersTable.rows.length,
			lastRow = usersTable.rows[rowsCount-1],
			input1 = lastRow.querySelector('input[name="userNum[]"]');

		if(input1 != null) {
			var
				lastIdStr = input1.value,

				// add required attribut for some input elements
				setRequiredInputs = function(parent, selector) {
					selector.split(' ').forEach(function(field) {
						var input = parent.querySelector('input[name$="_' + field + '"]');
						if(input != null) {
							input.setAttribute('required', true);
						}
					});
				},

				// update id users for every last rows
				setIdNewUsers = function() {
					var
						parent = lastRow.parentElement,
						newRows = parent.querySelectorAll('tr.users-csv'),
						newId = lastRow.lastId;

					for(var i=0, iMax=newRows.length; i<iMax; i++) {
						var
							row = newRows.item(i),
							newIdStr = ('00' + newId.toString()).substr(-3);
						if(! row.classList.contains('new')) {
							row.cells[1].innerHTML = newIdStr;
						}
						// Overwriting innerHTML loses events and changed values !
						var elements = row.querySelectorAll('input, select');
						for(var j=0, jMax=elements.length; j<jMax; j++) {
							var
								elmnt = elements.item(j);
							if(elmnt.name == 'userNum[]') {
								elmnt.value = newIdStr;
							}
							if(elmnt.hasAttribute('id')) {
								elmnt.id = elmnt.id.replace(/\d{3}_/, newIdStr+'_');
							}
							if(elmnt.hasAttribute('name')) {
								elmnt.name = elmnt.name.replace(/\d{3}_/, newIdStr+'_');
							}
						}
						newId++;
					}

				}

				deleteRow = function(event) {
					if(confirm(event.target.title)) {
						var
							row = this.parentElement,
							parent = row.parentElement;
						parent.removeChild(row);
						setIdNewUsers();
					}
				},

				// callback function when user clicks on the "new user" button
				addNewRow = function(event) {

					// the event is attached to the last row in the users table
					if(event.target.classList.contains('users-csv-btn')) {
						event.preventDefault();

						// this is the last row of the table
						// Add a new row in the users table before this and drop colspan="2" attribute
						var
							innerHTML = this.innerHTML,
							tr = document.createElement('TR'),
							td = document.createElement('TD'),
							oldCell0;
						tr.innerHTML = innerHTML;
						tr.className = 'users-csv';
						td.innerHTML = '<span class="delete-row" title="' + this.deleteTitle + '">&cross;</span>';
						this.parentElement.insertBefore(tr, this);
						oldCell0 = tr.cells[0];
						oldCell0.innerHTML = '000';
						oldCell0.classList.remove('users-csv-btn');
						oldCell0.removeAttribute('colSpan');
						tr.insertBefore(td, oldCell0);
						td.addEventListener('click', deleteRow);
						setRequiredInputs(tr, 'name login password');
						setIdNewUsers();
					}
				};

			if(! isNaN(lastIdStr)) {
				lastRow.lastId = parseInt(lastIdStr);
				lastRow.classList.add('users-csv'); // required for updating id new-users
				lastRow.table = usersTable;
				lastRow.addEventListener('click', addNewRow);
				lastRow.cells[0].classList.add('users-csv-btn');

				var emailCell = lastRow.querySelectorAll('input[name$="email"]');
				if((emailCell != null) && (emailCell.type != 'email')) {
					emailCell.type = 'email';
				}

				var
					deleteOption = document.querySelector('#' + idSelection+ ' option[value="delete"]');
				lastRow.deleteTitle = (deleteOption != null) ? deleteOption.textContent : 'delete';
			}
		}
	}

})('users-table', 'id_selection');

(function(idButton, idInputFile) {

	// export every user into a .CSV file
	var
		btn = document.getElementById(idButton),
		idTimer = null,
		restore = function() {
			btn.inputFile.removeAttribute('disabled');
			if(typeof(idTimer) == 'number') {
				console.log(idTimer);
				clearTimeout(idTimer);
			}
		};

	if(btn != null) {
		var inputFile = document.getElementById(idInputFile);
		if(inputFile != null) {
			btn.inputFile = inputFile;
			btn.addEventListener('click', function(event) {
				if(typeof(this.inputFile) == 'object') {
					this.inputFile.setAttribute('disabled', true);
					// downloading file attachment is running outside Javascript, So, fire a timer to return at initial state
					idTimer = setTimeout(restore, 2000);
				}
			});
		}
	}

})('users-csv-export', 'users-csv-file');

// create preview for the import csv file
(function(idButton, idInputFile, idLayer) {
	var
		button = document.getElementById(idButton),
		input = document.getElementById(idInputFile),
		layer = document.getElementById(idLayer);
	if((button != null) && (input != null) && (layer != null)) {

		var closeBtn = layer.querySelector('input[type="button"]');
		if(closeBtn != null) {

			closeBtn.layer = layer;
			closeBtn.addEventListener('click', function(event) {
				this.layer.classList.remove('display');
			});

			button.form.layer = layer;
			button.addEventListener('click', function(event) {
				var reader = new FileReader;
				reader.form = this.form;
				reader.addEventListener('load', function(event) {
					var table = this.form.layer.querySelector('table');
					if(table != null) {
						var
							buf = this.result,
							options = {
								fSep: this.form.elements['separator'].value,
								quot: this.form.elements['enclosure'].value
							},
							rows = [],
							datas;
						// 3 operating systems in the world, 3 end-of-lines. Cool !
						if(buf.match(/\r\n/) == null) {
							options.rSep = (buf.match(/\r/) == null) ? '\n' : '\r';
						}
						datas = buf.csvToArray(options),
						rows.push('<th>'+datas[0].join('</th>\n<th>')+'</th>\n');
						datas.slice(1,11).forEach(function(cells) {
							rows.push('<td>'+cells.join('</td>\n<td>')+'</td>\n');
						});
						table.innerHTML = '<tr>'+rows.join('</tr><tr>\n')+'</tr>';
						this.form.layer.classList.add('display');
					}
				});
				var file = this.form.elements['users-csv-file'].files[0];
				if(file) {
					reader.readAsText(file);
				}
			});

			input.previewBtn = button;
			input.addEventListener('change', function(event) {
				this.previewBtn.removeAttribute('disabled');
			});

		}
	}
})('users-csv-preview-btn', 'users-csv-file', 'users-csv-preview');

(function(idForm) {

	if(typeof(sessionStorage) == 'object') {
		var
			form = document.getElementById(idForm);
		if(form != null) {

			var
				store = function(event) {
					sessionStorage.setItem(this.name, this.value);
				},

				setSelected = function(select, value) {
					if(value != null) {
						for(var i=0, iMax=select.options.length; i<iMax; i++) {
							if(select.options[i].value == value) {
								select.selectedIndex = i;
								break;
							}
						}
					}
				},

				selects = form.querySelectorAll('select');

			if(selects.length > 0) {
				for(var i=0, iMax=selects.length; i<iMax; i++) {
					var item = selects.item(i);
					setSelected(item, sessionStorage.getItem(item.name));
					item.addEventListener('change', store);
				}
			}

		}
	}
})('form_users-csv');